library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity noveno_ensayo is
	port (
		clk : in std_logic;
 
		scl, sda : inout std_logic;
		d, d_n : out std_logic;
		error : out std_logic
	);
end noveno_ensayo;

architecture structural of noveno_ensayo is

	signal pwm_out : std_logic;
	signal pwm_n_out : std_logic;

	signal data_1 : std_logic_vector (11 downto 0);
 

	component pmod_adc_ad7991 is
		generic (
		sys_clk_freq : integer := 100_000_000); --input clock speed from user logic in Hz
		port (
			clk : in STD_LOGIC; --system clock
			reset_n : in STD_LOGIC; --asynchronous active-low reset
			scl : inout STD_LOGIC; --I2C serial clock
			sda : inout STD_LOGIC; --I2C serial data
			i2c_ack_err : out STD_LOGIC; --I2C slave acknowledge error flag
			adc_ch0_data : out STD_LOGIC_VECTOR(11 downto 0); --ADC Channel 0 data obtained
			adc_ch1_data : out STD_LOGIC_VECTOR(11 downto 0); --ADC Channel 1 data obtained
			adc_ch2_data : out STD_LOGIC_VECTOR(11 downto 0); --ADC Channel 2 data obtained
		adc_ch3_data : out STD_LOGIC_VECTOR(11 downto 0)); --ADC Channel 3 data obtained
	end component;
	
	component pwm_controller is
		port (
			clk : in std_logic;
			rst : in std_logic;
			duty_cycle : in std_logic_vector(11 downto 0);
			pwm_out : out std_logic;
			pwm_n_out : out std_logic
		);
	end component;

 
begin
	adc : pmod_adc_ad7991
	port map(
		clk => clk, --system clock
		reset_n => '1', 
		scl => scl, 
		sda => sda, 
		i2c_ack_err => error, 
		adc_ch0_data => data_1, 
		adc_ch1_data => open, 
		adc_ch2_data => open, 
		adc_ch3_data => open
	);
 
	pwm : pwm_controller
	port map(
		clk => clk, 
		rst => '0', 
		duty_cycle => data_1, 
		pwm_out => pwm_out, 
		pwm_n_out => pwm_n_out
	);

	d <= pwm_out;
	d_n <= pwm_n_out;
end architecture;